import __path from 'path';
import { fileURLToPath as __fileURLToPath } from 'url';
import { createRequire as __createRequire } from 'module';
const require = __createRequire(import.meta.url);
var __require = function(x) { return require(x); };
__require.__proto__.resolve = require.resolve;
const __filename = __fileURLToPath(import.meta.url);
const __dirname = __path.dirname(__filename);
var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));
var __require = /* @__PURE__ */ ((x) => typeof require !== "undefined" ? require : typeof Proxy !== "undefined" ? new Proxy(x, {
  get: (a, b) => (typeof require !== "undefined" ? require : a)[b]
}) : x)(function(x) {
  if (typeof require !== "undefined")
    return require.apply(this, arguments);
  throw new Error('Dynamic require of "' + x + '" is not supported');
});

// src/node/sveltePlugin.ts
import {
  svelte
} from "@sveltejs/vite-plugin-svelte";
import {
  virtualModuleRequestPath
} from "@vitebook/core/node";
import fs from "fs";

// src/node/markdoc.ts
import {
  escapeHtml,
  renderMarkdocToHTML
} from "@vitebook/core/node";
var svelteMarkdocTags = {
  svelte_head: {
    render: "svelte:head"
  }
};
var renderAttr = (_, name, value) => typeof value === "string" && !value.startsWith('"') ? `${name}="${value}"` : `${name}={${value}}`;
var renderMarkdoc = ({
  meta,
  content,
  imports,
  stuff
}) => {
  const markup = renderMarkdocToHTML(content, { attr: renderAttr });
  const scriptModule = [
    '<script context="module">',
    `  export const meta = ${JSON.stringify(meta)};`,
    "<\/script>"
  ].join("\n");
  const script = ["<script>", ...imports, "<\/script>"].join("\n");
  const svelteHead = stuff.head ? `<svelte:head>
${stuff.head}
</svelte:head>

` : "";
  return `${scriptModule}

${script}

${svelteHead}

${markup}`;
};
var codeNameRE = /^(code|Code)$/;
var fenceNameRE = /^(pre|Fence)$/;
var svelteHeadNameRE = /^svelte:head$/;
var transformTreeNode = ({
  node,
  stuff
}) => {
  if (node && typeof node !== "string") {
    const name = node.name;
    if (codeNameRE.test(name)) {
      escapeCodeContent(node);
    } else if (fenceNameRE.test(name)) {
      escapeFenceContent(node);
    } else if (svelteHeadNameRE.test(name)) {
      resolveSvelteHead(node, stuff);
    }
  }
};
function resolveSvelteHead(tag, stuff) {
  var _a;
  tag.attributes.__ignore = true;
  if (typeof tag.children[0] === "object") {
    if (Array.isArray((_a = tag.children[0]) == null ? void 0 : _a.children)) {
      stuff.head = tag.children[0].children.join("");
    }
  }
}
function escapeCodeContent(tag) {
  const isComponent = tag.name === "Code";
  const code = isComponent ? tag.attributes.code : tag.children[0];
  if (typeof code === "string") {
    if (isComponent) {
      tag.attributes.code = JSON.stringify(code);
    } else {
      tag.children[0] = htmlBlock(escapeHtml(code));
    }
  }
}
function escapeFenceContent(tag) {
  const isComponent = tag.name === "Fence";
  const code = isComponent ? tag.attributes.code : tag.children[0];
  const highlightedCode = tag.attributes.highlightedCode;
  if (isComponent) {
    tag.attributes.code = JSON.stringify(code);
    if (highlightedCode) {
      tag.attributes.highlightedCode = JSON.stringify(highlightedCode);
    }
  } else {
    tag.children[0] = htmlBlock(code);
  }
}
function htmlBlock(html) {
  return `{@html ${JSON.stringify(html)}}`;
}

// src/node/svelteSSRPlugin.ts
import MagicString from "magic-string";
var SVELTE_FILE_RE = /\.svelte($|\/)/;
function svelteSSRPlugin() {
  let app;
  return {
    name: "@vitebook/ssr",
    enforce: "post",
    vitebookInit(_app) {
      app = _app;
    },
    transform(code, id, { ssr } = {}) {
      if (ssr && !id.includes("@vitebook/svelte") && !id.includes("packages/svelte/dist/client") && SVELTE_FILE_RE.test(id)) {
        const mcs = new MagicString(code);
        const matchRE = /export\sdefault\s(.*?);/;
        const match = code.match(matchRE);
        const componentName = match == null ? void 0 : match[1];
        if (!match || !componentName)
          return null;
        const start = code.search(match[0]);
        const end = start + match[0].length;
        const addModuleCode = `  __vitebook__getSSRContext().modules.add(${JSON.stringify(app.dirs.root.relative(id))})`;
        mcs.overwrite(start, end, [
          "import { getSSRContext as __vitebook__getSSRContext } from '@vitebook/svelte';",
          `const $$render = ${componentName}.$$render;`,
          `${componentName}.$$render = function(...args) {`,
          addModuleCode,
          "  return $$render(...args)",
          "}",
          "",
          match[0]
        ].join("\n"));
        return {
          code: mcs.toString(),
          map: mcs.generateMap({ source: id }).toString()
        };
      }
      return null;
    }
  };
}

// src/node/sveltePlugin.ts
function sveltePlugin(config = {}) {
  let app;
  const clientEntry = __require.resolve(`@vitebook/svelte/entry-client.js`);
  const serverEntry = __require.resolve(`@vitebook/svelte/entry-server.js`);
  const isLocal = clientEntry.includes("packages/svelte/dist/client");
  return {
    name: "@vitebook/svelte",
    enforce: "pre",
    entry: {
      client: clientEntry,
      server: serverEntry
    },
    config() {
      return {
        server: {
          fs: {
            strict: !isLocal
          }
        },
        plugins: [svelteSSRPlugin()]
      };
    },
    vitebookConfig(config2) {
      config2.markdown.markdoc.tags = __spreadValues(__spreadValues({}, config2.markdown.markdoc.tags), svelteMarkdocTags);
      config2.markdown.render = renderMarkdoc;
      config2.markdown.transformTreeNode.push(transformTreeNode);
    },
    vitebookInit(_app) {
      var _a, _b, _c, _d;
      app = _app;
      const hasSveltePlugin = (_b = (_a = app.vite) == null ? void 0 : _a.config.plugins) == null ? void 0 : _b.flat().some((plugin) => plugin && plugin.name === "vite-plugin-svelte");
      if (!hasSveltePlugin) {
        app.plugins.push(svelte(__spreadProps(__spreadValues({}, config.svelte), {
          extensions: [
            ".svelte",
            ".md",
            ...((_c = config.svelte) == null ? void 0 : _c.extensions) ?? []
          ],
          compilerOptions: __spreadProps(__spreadValues({}, (_d = config.svelte) == null ? void 0 : _d.compilerOptions), {
            hydratable: true
          })
        })));
      }
    },
    resolveId(id) {
      if (id === virtualModuleRequestPath.app) {
        const path = app.dirs.pages.resolve("@app.svelte");
        return fs.existsSync(path) ? { id: path } : { id: __require.resolve("@vitebook/svelte/App.svelte") };
      }
      return null;
    }
  };
}

// src/node/index.ts
var node_default = sveltePlugin;
export {
  node_default as default,
  sveltePlugin
};
