import type { ClientLayout as ClientLayout_2 } from '@vitebook/core';
import type { ClientPage as ClientPage_2 } from '@vitebook/core';
import type { ClientPageModule as ClientPageModule_2 } from '@vitebook/core';
import type { LoadedClientLayout as LoadedClientLayout_2 } from '@vitebook/core';
import type { LoadedClientPage as LoadedClientPage_2 } from '@vitebook/core';
import { MarkdownFrontmatter } from '@vitebook/core';
import { MarkdownMeta } from '@vitebook/core';
import { Readable } from 'svelte/store';
import { Subscriber } from 'svelte/store';
import type { SvelteComponent } from 'svelte';
import { SvelteComponentTyped } from 'svelte';
import { Unsubscriber } from 'svelte/store';
import { Updater } from 'svelte/store';
import type { VitebookSSRContext } from '@vitebook/core';
import { Writable } from 'svelte/store';

export declare type ClientLayout = ClientLayout_2<SvelteModule>;

/** @typedef {typeof __propDef.props}  ClientOnlyProps */
/** @typedef {typeof __propDef.events}  ClientOnlyEvents */
/** @typedef {typeof __propDef.slots}  ClientOnlySlots */
export declare class ClientOnly extends SvelteComponentTyped<{}, {
    [evt: string]: CustomEvent<any>;
}, {
    default: {};
}> {
}

export declare type ClientPage = ClientPage_2<ClientPageModule>;

export declare type ClientPageModule = ClientPageModule_2<SvelteConstructor>;

export declare type ConfigureRouter = (router: Router) => void | Promise<void>;

export declare function createMemoryHistory(): MemoryHistory;

export declare const frontmatter: Readable<MarkdownFrontmatter>;

export declare function getRouter(): Router;

export declare function getSSRContext(): VitebookSSRContext;

export declare type GoToRouteOptions = {
    scroll?: false | {
        x: number;
        y: number;
    };
    replace?: boolean;
    keepfocus?: boolean;
    state?: any;
};

export declare const layouts: Readable<ClientLayout[]>;

export declare type LoadedClientLayout = LoadedClientLayout_2<SvelteModule>;

export declare type LoadedClientMarkdownPage = LoadedClientPage & {
    readonly meta: MarkdownMeta;
};

export declare type LoadedClientPage = LoadedClientPage_2<ClientPageModule>;

export declare type LoadedRouteLocation = RouteLocation & {
    component: RouteComponent;
};

export declare const markdown: Readable<MarkdownMeta | undefined>;

/** Used server-side.  */
export declare class MemoryHistory implements History {
    scrollRestoration: ScrollRestoration;
    protected history: string[];
    protected position: number;
    protected _state: {};
    get state(): {};
    get length(): number;
    back(): void;
    forward(): void;
    go(delta: number): void;
    pushState(data: any, _: string, url?: string | URL | null): void;
    replaceState(state: any, _: string, url?: string | URL | null): void;
    protected setLocation(location: string): void;
}

export declare type NavigationOptions = {
    url: URL;
    scroll?: false | {
        x: number;
        y: number;
    };
    keepfocus?: boolean;
    hash?: string;
};

export declare const page: PageStore;

/** @typedef {typeof __propDef.props}  PageLayoutsProps */
/** @typedef {typeof __propDef.events}  PageLayoutsEvents */
/** @typedef {typeof __propDef.slots}  PageLayoutsSlots */
export declare class PageLayouts extends SvelteComponentTyped<{
    index?: number | undefined;
}, {
    [evt: string]: CustomEvent<any>;
}, {
    default: {};
}> {
}

export declare const pages: Readable<ClientPage[]>;

export declare type PageStore = {
    subscribe: Writable<LoadedClientPage>['subscribe'];
    __update: Writable<LoadedClientPage>['update'];
    __set: Writable<LoadedClientPage>['set'];
};

/** @typedef {typeof __propDef.props}  PageViewProps */
/** @typedef {typeof __propDef.events}  PageViewEvents */
/** @typedef {typeof __propDef.slots}  PageViewSlots */
export declare class PageView extends SvelteComponentTyped<{}, {
    [evt: string]: CustomEvent<any>;
}, {}> {
}

export declare type Route = {
    path: string;
    redirect?: string;
    prefetch?: RoutePrefetch;
    loader: RouteLoader;
};

export declare const route: {
    set: undefined;
    update: undefined;
    __update: (this: void, updater: Updater<LoadedRouteLocation>) => void;
    __set: (this: void, value: LoadedRouteLocation) => void;
    subscribe(this: void, run: Subscriber<LoadedRouteLocation>, invalidate?: ((value?: LoadedRouteLocation | undefined) => void) | undefined): Unsubscriber;
};

export declare type RouteComponent = SvelteConstructor | {
    adapter: SvelteConstructor;
    component: unknown;
};

export declare type RouteLoader = (location: RouteLocation) => RouteComponent | Promise<RouteComponent> | RouteLoaderComponent | Promise<RouteLoaderComponent>;

export declare type RouteLoaderComponent = SvelteConstructor | SvelteModule;

export declare type RouteLocation = Route & {
    id: string;
    url: URL;
};

export declare type RoutePrefetch = (location: RouteLocation) => void | Promise<void>;

export declare class Router {
    protected readonly history: History;
    protected readonly routes: Map<string, Route>;
    enabled: boolean;
    baseUrl: string;
    scrollBehaviour?: ScrollBehavior;
    scrollOffset: () => {
        top: number;
        left: number;
    };
    beforeNavigate?: (url: URL) => void | Promise<void>;
    afterNavigate?: (url: URL) => void | Promise<void>;
    protected _isReady: boolean;
    protected _resolveReadyPromise?: () => void;
    protected _isReadyPromise: Promise<unknown>;
    get isReady(): boolean;
    get waitUntilReady(): Promise<unknown>;
    constructor({ baseUrl, history, routes }: RouterOptions);
    hasRoute(path: string): boolean;
    addRoute(route: Route): void;
    removeRoute(route: string | Route): void;
    owns(url: URL): boolean;
    findRoute(url: URL): RouteLocation | undefined;
    back(): void;
    go(href: string, { scroll, replace, keepfocus, state, }?: GoToRouteOptions): Promise<unknown>;
    prefetch(url: URL): Promise<void>;
    protected navigate({ url, keepfocus, scroll, hash, }: NavigationOptions): Promise<void>;
    protected scrollToPosition({ scroll, hash, }: Pick<NavigationOptions, 'scroll' | 'hash'>): Promise<void>;
    initListeners(): void;
}

export declare const ROUTER_CTX_KEY: unique symbol;

export declare type RouterOptions = {
    baseUrl: string;
    history: History;
    routes?: Route[];
};

export declare const SSR_CTX_KEY: unique symbol;

export declare type SvelteConstructor = typeof SvelteComponent;

export declare type SvelteModule = {
    readonly default: SvelteConstructor;
};

export { }
